"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var chalk_1 = require("chalk");
var utils_1 = require("../utils");
var validators_1 = require("../validators");
var filters_1 = require("../filters");
var commit_template_1 = require("../commit-template");
function validatorFactory(rules) {
    return function (value, answers) {
        var headerValue = commit_template_1.headerTemplate(answers.type, answers.scope, value);
        return validators_1.validate([
            {
                value: headerValue,
                rule: rules['header-max-length'],
                validator: validators_1.maxLengthValidator,
                message: function (length) { return "Header \"" + headerValue + "\" cannot be longer than " + length; }
            },
            {
                value: value,
                rule: rules['subject-max-length'],
                validator: validators_1.maxLengthValidator,
                message: function (length) { return "Subject maximum length of " + length + " has been exceeded"; }
            },
            {
                value: value,
                rule: rules['subject-min-length'],
                validator: validators_1.minLengthValidator,
                message: function (length) { return "Subject minimum length of " + length + " has not been met"; }
            },
            {
                value: value,
                rule: rules['subject-empty'],
                validator: validators_1.emptyValidator,
                message: function () { return 'Subject cannot be empty'; }
            },
            {
                value: value,
                rule: rules['subject-case'],
                validator: validators_1.caseValidator,
                message: function (ruleValue, applicable) { return "Subject must " + (applicable == 'never' ? 'not ' : '') + "be in " + ruleValue; }
            }
        ]);
    };
}
exports.validatorFactory = validatorFactory;
function filterFactory(rules) {
    return function (value) {
        return utils_1.pipeWith(value, function (v) { return filters_1.wordCaseFilter(v, rules['subject-case']); }, function (v) { return filters_1.fullStopFilter(v, rules['subject-full-stop']); });
    };
}
exports.filterFactory = filterFactory;
function messageFactory(rules) {
    return function (answers) {
        var maxLength = utils_1.valueFromRule(rules['header-max-length']);
        if (!maxLength) {
            return "Write a short, imperative tense description of the change:\n";
        }
        return "Write a short, imperative tense description of the change (max " + (maxLength -
            commit_template_1.headerTemplate(answers.type, answers.scope).length) + " chars):\n";
    };
}
exports.messageFactory = messageFactory;
function transformerFactory(rules) {
    var filter = filterFactory(rules);
    return function (value, answers) {
        var headerMaxLength = utils_1.valueFromRule(rules['header-max-length']);
        if (headerMaxLength) {
            var color = filter(value).length <= headerMaxLength - commit_template_1.headerTemplate(answers.type, answers.scope).length ? chalk_1.green : chalk_1.red;
            return color("(" + value.length + ") " + value);
        }
        var subjectMaxLength = utils_1.valueFromRule(rules['subject-max-length']);
        if (subjectMaxLength) {
            var color = filter(value).length <= subjectMaxLength ? chalk_1.green : chalk_1.red;
            return color("(" + value.length + ") " + value);
        }
        return value;
    };
}
exports.transformerFactory = transformerFactory;
function subjectMaker(questions, rules) {
    var filter = filterFactory(rules);
    var question = {
        message: messageFactory(rules),
        name: 'subject',
        type: 'input',
        validate: validatorFactory(rules),
        filter: filter,
        transformer: transformerFactory(rules)
    };
    return tslib_1.__spreadArrays(questions, [question]);
}
exports.subjectMaker = subjectMaker;
